package de.jaetzold.art.examples;
// ...
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
// ...
public class AriadneSimpleAlgorithm implements Ariadne {
	protected NormalizedDriveTrain driveTrain;
	protected int leftDirection;
	protected double orientationAngle;
	public AriadneSimpleAlgorithm(Robot robot) {
		this.driveTrain = robot.getNormalizedDriveTrain();
		this.leftDirection = robot.getLeftDirection();
		this.orientationAngle = robot.getOrientationAngle();
		setState(ORIENTED);
	}
// BEGIN ariadne:impl:orient
	public void orientByAngle(double angle) {
		setState(ORIENTING);
		if((angle - orientationAngle)*leftDirection > 0) {
			driveTrain.leftSpin(Math.abs(angle - orientationAngle));
		} else if((angle - orientationAngle)*leftDirection < 0) {
			driveTrain.rightSpin(Math.abs(angle - orientationAngle));
		}
		setState(ORIENTED);
	}
// END ariadne:impl:orient
	public interface Robot {
		public NormalizedDriveTrain getNormalizedDriveTrain();
		public int getLeftDirection();
		public double getOrientationAngle();
	}
// ...
	protected void setState(int state) {
		int oldState = this.state;
		this.state = state;
		changes.firePropertyChange("state", oldState, state);
	}
	
	protected PropertyChangeSupport changes =
				new PropertyChangeSupport(this);
	public void addPropertyChangeListener(PropertyChangeListener l) {
		changes.addPropertyChangeListener(l);
	}
	public void removePropertyChangeListener(PropertyChangeListener l) {
		changes.removePropertyChangeListener(l);
	}
	
	private int state;
	public int getState() {
		return state;
	}
// ...
}
