package de.jaetzold.art.examples;

import de.jaetzold.util.StateDecider;
import de.jaetzold.util.TransitionMatrixStateDecider;

/**	This one assumes that a direction-change only happens after two concurrent Rotation-sensor states have been the same.
	*/
public class RCXRotationFasterCountStateDecider implements StateDecider {
	public boolean isCacheable() {
		return false;
	}
	private static final int x = 0;
	private StateDecider countMatrix =
			new TransitionMatrixStateDecider(
						/*	turning:    stopped  |  negative  |  positive	*/
			new int[][]{/*  l\n   0,  1, 2, 3, 4,  5, 6, 7, 8,  9,10,11,12	*/
						/*	0*/	{ x,  0, 0, 0, 0, -1,-1,-1,-1,  1, 1, 1, 1},

						/*	1*/	{ x,  0, x, x, x,  0,-3,-2,-1,  0, 3, 2, 1},
						/*	2*/	{ x,  x, 0, x, x, -1, 0,-3,-2,  1, 0, 3, 2},
		/* stopped */	/*	3*/	{ x,  x, x, 0, x, -2,-1, 0,-3,  2, 1, 0, 3},
						/*	4*/	{ x,  x, x, x, 0, -3,-2,-1, 0,  3, 2, 1, 0},

						/*	5*/	{ x,  0,-3,-2,-1,  x,-3,-2,-1,  x,-3,-2,-1},
						/*	6*/	{ x, -1, 0,-3,-2, -1, x,-3,-2, -1, x,-3,-2},
		/* negative */	/*	7*/	{ x, -2,-1, 0,-3, -2,-1, x,-3, -2,-1, x,-3},
						/*	8*/	{ x, -3,-2,-1, 0, -3,-2,-1, x, -3,-2,-1, x},

						/*	9*/	{ x,  0, 1, 2, 3,  x, 1, 2, 3,  x, 1, 2, 3},
						/* 10*/	{ x,  3, 0, 1, 2,  3, x, 1, 2,  3, x, 1, 2},
		/* positive */	/* 11*/	{ x,  2, 3, 0, 1,  2, 3, x, 1,  2, 3, x, 1},
						/* 12*/	{ x,  1, 2, 3, 0,  1, 2, 3, x,  1, 2, 3, x},
						});

	// CBD: +'s dependant from the something
	// like a motor, or the next direction
	private StateDecider turnStateMatrix =
			new TransitionMatrixStateDecider(
				new int[][]{/*  l\n   0, 1, 2, 3, 4	*/
							/*	0*/	{ 0, 1, 2, 3, 4},	//
							
							/*	1*/	{ 0, 1,10,+3, 8},	// -\
							/*	2*/	{ 0, 5, 2,11,+4},	//	  turning
							/*	3*/	{ 0,+1, 6, 3,12},	//	  stopped
							/*	4*/	{ 0, 9,+2, 7, 4},	// -/
							
							/*	5*/	{ 0, 1, 6, 7, 8},	// -\
							/*	6*/	{ 0, 5, 2, 7, 8},	//	  turning
							/*	7*/	{ 0, 5, 6, 3, 8},	//	  negative
							/*	8*/	{ 0, 5, 6, 7, 4},	// -/
							
							/*	9*/	{ 0, 1,10,11,12},	// -\
							/* 10*/	{ 0, 9, 2,11,12},	//	  turning
							/* 11*/	{ 0, 9,10, 3,12},	//	  positive
							/* 12*/	{ 0, 9,10,11, 4}});	// -/

	private double lastTurnState;
	private boolean initializedState;
	public synchronized double nextStateValue(	double lastState,
												double newState)
	{
		if(!initializedState) {
			lastTurnState = lastState;
			initializedState = true;
		}
		double newTurnState =
					turnStateMatrix.nextStateValue(lastState, newState);
		double result = countMatrix.nextStateValue(lastTurnState, newTurnState);
		lastTurnState = newTurnState;
		return result;
	}
}
