package de.jaetzold.art.examples;

import de.jaetzold.util.RangeStateDecider;
import de.jaetzold.util.StateDecider;
import de.jaetzold.util.SingleRange;
import de.jaetzold.util.Range;

/** A simple class which maps raw sensor-values to one of the four positions  the LEGO-Rotationsensor may have.
	The raw sensor values are around 359, 527, 1023 and 777 which are mapped by this decider to the states 1, 2, 3 and 4. All values not very close to the four named (about +-7) don't result in a state-change.
	It has such short borders because it seems that the original sensor measuring of the RCX also doesn't take any "wrong" values into account.
	*/
// BEGIN rotation:state
public class RCXRotationSensorStateDecider implements StateDecider {
	protected StateDecider target;
	
	public RCXRotationSensorStateDecider() {
		target = new RangeStateDecider(
						   new Range[] {null,
										new SingleRange(352,366),
										new SingleRange(520,535),
										new SingleRange(1020,1024),
										new SingleRange(770,785)});
	}
	
	public double nextStateValue(double actualState, double value) {
		double newState = target.nextStateValue(actualState, value);
		if(newState == 0) {
			// The standard rcx-interpretation of such
			// 'false' values seems to simply ignore them
			return actualState;
		} else {
			return newState;
		}
	}
// ...	
	public boolean isCacheable() {
		return false;	// because values not in the ranges have actualState as result
	}
// ...
}
// END rotation:state
