package de.jaetzold.art.examples;

import de.jaetzold.art.BooleanSensor;
import de.jaetzold.art.SensorListener;
import de.jaetzold.art.SensorEvent;

public class SimpleTrusty {
	protected DriveTrain chassis;
	protected int backupTime;
	protected int turnTime;
	
	static interface Robot {
		public DriveTrain getDriveTrain();
		public BooleanSensor getLeftBumper();
		public BooleanSensor getRightBumper();
		public int getBackupTime();
		public int getTurnTime();
	}

	public SimpleTrusty(Robot robot) {
		chassis = robot.getDriveTrain();
		
		backupTime = robot.getBackupTime();
		turnTime = robot.getTurnTime();
		
		final BooleanSensor leftBumper = robot.getLeftBumper();
		leftBumper.addSensorListener(
			new SensorListener() {
				public void processEvent(SensorEvent se) {
					if(leftBumper.convertToBoolean(se.getValue())) {
						bumpedLeft();
					}
				}
			}
		);
		
		final BooleanSensor rightBumper = robot.getRightBumper();
		rightBumper.addSensorListener(
			new SensorListener() {
				public void processEvent(SensorEvent se) {
					if(rightBumper.convertToBoolean(se.getValue())) {
						bumpedRight();
					}
				}
			}
		);
		
		// start moving
		chassis.forward();
	}
	
	public void bumpedLeft() {
		chassis.backward();
		try {
			Thread.sleep(backupTime);
		} catch(InterruptedException ie) {
		}
		
		chassis.rightSpin();
		try {
			Thread.sleep(3*turnTime/4 +(int)(Math.random()*turnTime/2));
		} catch(InterruptedException ie) {
		}
		
		chassis.forward();
	}
	
	public void bumpedRight() {
		chassis.backward();
		try {
			Thread.sleep(backupTime);
		} catch(InterruptedException ie) {
		}
		
		chassis.leftSpin();
		try {
			Thread.sleep(3*turnTime/4 +(int)(Math.random()*turnTime/2));
		} catch(InterruptedException ie) {
		}
		
		chassis.forward();
	}
}
