package de.jaetzold.art.examples;

import de.jaetzold.art.LightSensor;
import de.jaetzold.art.RobotInterfaceFactory;
import de.jaetzold.art.RobotInterface;
import de.jaetzold.art.Port;

public class SubsumptionLiSeMSDemoRobot implements SubsumptionLiSe.Robot {
	protected LiSe lise;
	protected LightSensor leftSensor;
	protected LightSensor rightSensor;
	
	public SubsumptionLiSeMSDemoRobot(LiSe lise, RobotInterface iface) {
		this.lise = lise;
		
		Port leftPort = iface.getPort("2");
		Port rightPort = iface.getPort("1");
		if(rightPort == null || leftPort == null) {
			throw new IllegalArgumentException("RobotInterface " +iface
												+" provides not the required Ports."
												+"2=" +leftPort
												+", 1=" +rightPort);
		}
		
		leftSensor = new LightSensor();
		rightSensor = new LightSensor();
		leftSensor.connectWith(leftPort);
		rightSensor.connectWith(rightPort);
		
		// Lego's light sensor can also measure normal room-light level,
		// but we need only flashlight
		leftSensor.setMeasurableMin(350);
		rightSensor.setMeasurableMin(350);
		
		// depends on the precision of the Servo, but also on the change
		// in light-value if the Servo isn't properly positioned
		leftSensor.setPrecision(leftSensor.getPrecision()*2);
		rightSensor.setPrecision(rightSensor.getPrecision()*2);
	}
	
	public LiSe getLiSe() {
		return lise;
	}
	public LightSensor getLeftSensor() {
		return leftSensor;
	}
	public LightSensor getRightSensor() {
		return rightSensor;
	}
// ...
	private static String className =
			"de.jaetzold.art.examples.SubsumptionLiSeMSDemoRobot";
	public static void main(String[] argv) throws Exception {
		RobotInterface iface = new RobotInterfaceFactory().getInterface("MS");
		
		LiSe lise = new LiSeSimpleAlgorithm(
						new LiSeSimpleMSDemoRobot(iface));
		
		SubsumptionLiSe subsumptionLise =
					new SubsumptionLiSe(
						new SubsumptionLiSeMSDemoRobot(lise, iface));
		subsumptionLise.getSchedulerPerformer().start();// start subsumption
		
		LiSeView view = new LiSeView(subsumptionLise);
		LiSeCommands commands = new LiSeCommands(subsumptionLise);
		
		java.awt.Frame f = new java.awt.Frame(className);
		f.setLayout(new java.awt.BorderLayout());
		f.add("North", view);
		f.add("South", commands);
		
		f.addWindowListener(
			new java.awt.event.WindowAdapter() {
				public void windowClosing(java.awt.event.WindowEvent we) {
					System.exit(0);
				}
			}
		);
		f.pack();
		f.show();
	}
// ...
}
	
