package de.jaetzold.art.examples;

import de.jaetzold.art.RobotInterface;
import de.jaetzold.art.BooleanSensor;
import de.jaetzold.art.RawSensor;
import de.jaetzold.art.Port;
import de.jaetzold.util.SingleRange;

public class SubsumptionTrustyMSDemoRobot extends SubsumptionTrustyBaseRobot {
	public SubsumptionTrustyMSDemoRobot(RobotInterface iface) {
		this(new TrustySimpleAlgorithm(
				new TrustySimpleMSDemoRobot(iface)
			), iface);
	}
// BEGIN SubTrusty:robot:ms
	public SubsumptionTrustyMSDemoRobot(Trusty trusty,
										RobotInterface iface)
	{
		super(trusty, iface);
		
		// switch left and right
		BooleanSensor tmp = leftSensor;
		leftSensor = rightSensor;
		rightSensor = tmp;
		
		// in anticipation of LiSe the BooleanSensor's have to be
		// configured to handle the Raw value since they are connected
		// to the same ports as the LightSensor's of LiSe
		Port leftPort = leftSensor.getPort();
		Port rightPort = rightSensor.getPort();
		leftSensor.disconnect();
		rightSensor.disconnect();
		
		// put a RawSensor in-between
		RawSensor leftRaw = new RawSensor();
		RawSensor rightRaw = new RawSensor();
		leftRaw.connectWith(leftPort);
		rightRaw.connectWith(rightPort);
		leftSensor.connectWith(leftRaw.getSensorPort());
		rightSensor.connectWith(rightRaw.getSensorPort());
		
		// first Range is for 0, second for 1 and third is hysteresis
		// these are non-standard RCX-values because Light-Sensors
		// can deliver values as low as 320
		leftSensor.setStateDecider(
			new RCXRawSwitchStateDecider(
				new SingleRange(220),
				new SingleRange(0, 190),
				new SingleRange(190, 220)
			));
		rightSensor.setStateDecider(
			new RCXRawSwitchStateDecider(
				new SingleRange(220),
				new SingleRange(0, 190),
				new SingleRange(190, 220)
			));
		// a different contructor could be nice here, which only needs
		// one Range which is the hysteresis
	}
// END SubTrusty:robot:ms
// ...
	public SubsumptionTrustyMSDemoRobot() throws Exception {
		this(new de.jaetzold.art.RobotInterfaceFactory().getInterface(
				System.getProperty(className +".portName", "MS")
			)
		);
	}
	private static String className =
			"de.jaetzold.art.examples.SubsumptionTrustyMSDemoRobot";
	public static void main(String[] argv) throws Exception {
		if(argv.length > 0) {
			System.setProperty(className +".portName", argv[0]);
		}
		SubsumptionTrusty.Robot robot = new SubsumptionTrustyMSDemoRobot();
		SubsumptionTrusty trusty = new SubsumptionTrusty(robot);
		trusty.getSchedulerPerformer().start();	// start subsumption
		
		TrustyView view = new TrustyView(trusty);
		
		TrustyCommands commands = new TrustyCommands(trusty);
		
		java.awt.Frame f = new java.awt.Frame(className);
		f.setLayout(new java.awt.BorderLayout());
		f.add("North", view);
		f.add("South", commands);
		
		f.addWindowListener(
			new java.awt.event.WindowAdapter() {
				public void windowClosing(java.awt.event.WindowEvent we) {
					System.exit(0);
				}
			}
		);
		f.pack();
		f.show();
	}
// ...
}
	
