// BEGIN Trusty:init
package de.jaetzold.art.examples;
// ...
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
// ...
public class TrustySimpleAlgorithm implements Trusty {
	protected NormalizedDriveTrain driveTrain;
	protected double backupMeters;
	protected double turnAngle;

	public TrustySimpleAlgorithm(Robot robot) {
		driveTrain = robot.getNormalizedDriveTrain();
		backupMeters = robot.getBackupMeters();
		turnAngle = robot.getTurnAngle();
	}
	public static interface Robot {
		public NormalizedDriveTrain getNormalizedDriveTrain();
		public double getBackupMeters();
		public double getTurnAngle();
	}
// BEGIN Trusty:impl
	public void avoidLeft() {
		setState(AVOID_LEFT);
		driveTrain.backward(backupMeters);
		driveTrain.rightSpin(turnAngle);
		stop();
	}
// END Trusty:impl
// ...
	public void avoidRight() {
		setState(AVOID_RIGHT);
		driveTrain.backward(backupMeters);
		driveTrain.leftSpin(turnAngle);
		stop();
	}
	public void stop() {
		setState(STOP);
		driveTrain.stop();
	}
	public void move() {
		setState(MOVE);
		driveTrain.forward();
	}
	
	protected void setState(int state) {
		int oldState = this.state;
		this.state = state;
		changes.firePropertyChange("state", oldState, state);
	}
	
	protected PropertyChangeSupport changes =
				new PropertyChangeSupport(this);
	public void addPropertyChangeListener(PropertyChangeListener l) {
		changes.addPropertyChangeListener(l);
	}
	public void removePropertyChangeListener(PropertyChangeListener l) {
		changes.removePropertyChangeListener(l);
	}
	
	private int state;
	public int getState() {
		return state;
	}
	
	public TrustySimpleAlgorithm() throws Exception {
		this((Robot)Class.forName(
				System.getProperty(
					className +".robotClass",
					"de.jaetzold.art.examples.TrustySimpleBaseRobot"
				)
			).newInstance()
		);
	}
	private static String className =
			"de.jaetzold.art.examples.TrustySimpleAlgorithm";
	public static void main(String[] argv) throws Exception {
		int index = 0;
		if(argv.length > index) {
			System.setProperty(className +".robotClass", argv[index++]);
		}
		TrustySimpleAlgorithm trusty = new TrustySimpleAlgorithm();
		
		TrustyView view = new TrustyView(trusty);
		TrustyCommands commands = new TrustyCommands(trusty);
		
		java.awt.Frame f = new java.awt.Frame(className);
		f.setLayout(new java.awt.BorderLayout());
		f.add("North", view);
		f.add("South", commands);
		
		f.addWindowListener(
			new java.awt.event.WindowAdapter() {
				public void windowClosing(java.awt.event.WindowEvent we) {
					System.exit(0);
				}
			}
		);
		f.pack();
		f.show();
	}
// ...
}
// END Trusty:init
